@@ -1,12 +1,13 @@
 package ZooKeeper;
 use ZooKeeper::XS;
 use ZooKeeper::Constants;
-use Digest::SHA qw(sha1_base64);
 use Carp;
+use Digest::SHA qw(sha1_base64);
+use Module::Runtime qw(require_module);
 use Moo;
 use 5.10.1;
 
-our $VERSION = '0.0.11';
+our $VERSION = '0.0.12';
 
 =head1 NAME
 
@@ -129,7 +130,7 @@ has timeout => (
 
 =head2 watcher
 
-A subroutine reference to be called by a default watcher for the ZooKeeper session.
+A subroutine reference to be called by a default watcher for ZooKeeper session events.
 
 =cut
 
@@ -160,6 +161,12 @@ has buffer_length => (
     default => 2048,
 );
 
+=head2 client_id
+
+The client_id for a ZooKeeper session. Can be set during construction to resume a previous session.
+
+=cut
+
 =head2 dispatcher
 
 The implementation of ZooKeeper::Dispatcher to be used. Defaults to AnyEvent.
@@ -176,41 +183,46 @@ Valid types include:
 
 =cut
 
-has dispatcher_type => (
-    is       => 'ro',
-    init_arg => 'dispatcher',
-    default  => sub { $ENV{PERL_ZOOKEEPER_DISPATCHER} || 'AnyEvent' },
+has dispatcher => (
+    is      => 'ro',
+    default => sub { $ENV{PERL_ZOOKEEPER_DISPATCHER} || 'AnyEvent' },
 );
 
-has dispatcher => (
-    is       => 'rw',
+has _dispatcher_obj => (
+    is       => 'ro',
     init_arg => undef,
+    lazy     => 1,
+    builder  => '_build_dispatcher_obj',
     handles  => [qw(create_watcher wait)],
 );
 
-sub _build_dispatcher {
+sub _build_dispatcher_obj {
     my ($self) = @_;
-    my $type  = lc $self->dispatcher_type;
+    my $type  = lc $self->dispatcher;
     my $class = $type eq 'anyevent'  ? 'ZooKeeper::Dispatcher::AnyEvent'
+
               : $type eq 'interrupt' ? 'ZooKeeper::Dispatcher::Interrupt'
               : croak "Unrecognized dispatcher type: $type";
 
-    unless (eval "require $class; 1") {
-        croak "Could not require dispatcher class $class: $@";
-    }
-
-    return $class->new;
+    require_module($class);
+    return $class->new(ignore_session_events => $self->ignore_session_events);
 }
 
-=head2 client_id
+=head2 ignore_session_events
 
-The client_id for a ZooKeeper session. Can be set during construction to resume a previous session.
+If set to false, all watchers will be triggered for session events, such as disconnecting and reconnecting to the ZooKeeper server. This means that watchers can be triggered multiple times, until the watcher event is triggered.
+
+The default value is true, which will only trigger watchers once, for the watcher event.
 
 =cut
 
+has ignore_session_events => (
+    is      => "ro",
+    default => 1,
+);
+
 sub BUILD {
     my ($self, $args) = @_;
-    $self->dispatcher($self->_build_dispatcher);
     my $default_watcher = $self->watcher ? $self->create_watcher('' => $self->watcher, type => 'default') : undef;
 
     $self->_xs_init($self->hosts, $self->timeout, $default_watcher, $args->{client_id});
